// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                            ADVANCED TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */
(function () {
    // ================================ START: CONSTRUCTORS  ====================================================  
    // Constructor call for mmWaveAdvanced object:
    function mmWaveAdvanced() {
        if (!(this instanceof mmWaveAdvanced))
            return new mmWaveAdvanced();
        this.init();
    }
    
    // Initialize the mmWaveAdvanced object:
    function init() {
        this.Input = Adv_Globals.BackendInputs;
        this.Output = Adv_Globals.BackendOutputs;
    }
    // ================================ END: CONSTRUCTORS  ==================================================================
    // ================================ START: UPDATE FUNCTIONS  ============================================================ 
    var updateOutputComputation = function(advObject){
        // Grab the relevant contexts:
        var Input   = advObject.Input;
        var Output  = advObject.Output;

        Output.sdkVersion = get_supported_SDK(Input.device);
        
        // Calculate all the output values from the inputs:
        Output.adcbufCfg_adcOutputFmt               = get_adcbufCfg_adcOutputFmt(Input.adcMode);

        // Chirp Parameters
        Output.chirpParam_startFrequency            = get_chirpParam_startFrequency(Output.sdkVersion, Input.profileCfg_startFreq, Input.chripTimingCfg_ChirpRfFreqStart);
        Output.chirpParam_frequencySlope            = get_chirpParam_frequencySlope(Output.sdkVersion, Input.profileCfg_freqSlope, Input.chripTimingCfg_ChirpRfFreqSlope);
        Output.chirpParam_samplingRate              = get_chirpParam_samplingRate(Output.sdkVersion, Input.profileCfg_digOutSampleRate, Input.chirpComnCfg_DigOutputSampRate);
        Output.chirpParam_numSamples                = get_chirpParam_numSamples(Output.sdkVersion, Input.profileCfg_numAdcSamples, Input.chirpComnCfg_NumOfAdcSamples);
        Output.chirpParam_numChirpLoops             = get_chirpParam_numChirpLoops(Output.sdkVersion, Input.frameCfg_numLoops, Input.frameCfg5_NumOfBurstsInFrame, Input.frameCfg5_NumOfChirpsInBurst);
        Output.chirpParam_idleTime                  = get_chirpParam_idleTime(Output.sdkVersion, Input.profileCfg_idleTime, Input.chripTimingCfg_ChirpIdleTime);
        Output.chirpParam_adcValidStartTime         = get_chirpParam_adcValidStartTime(Output.sdkVersion, Input.profileCfg_adcStartTime, Output.chirpParam_samplingRate, Input.chirpTimingCfg_ChirpAdcSkipSamples);
        Output.chirpParam_rampEndTime               = get_chirpParam_rampEndTime(Output.sdkVersion, Input.profileCfg_rampEndTime, Input.chirpComnCfg_ChirpRampEndTime);
        Output.chirpParam_framePeriodicity          = get_chirpParam_framePeriodicity(Output.sdkVersion, Input.frameCfg_framePeriodicity, Input.frameCfg5_FramePeriodicity);

        // Device & Board Specific Parameters
        Output.deviceBoardParam_numRxAntennas       = get_deviceBoardParam_numRxAntennas(Output.sdkVersion, Input.channelCfg_rxChannelEn, Input.channelCfg5_rxChannelEn);
        Output.deviceBoardParam_numTxAntennas       = get_deviceBoardParam_numTxAntennas(Output.sdkVersion, Input.channelCfg_txChannelEn, Input.channelCfg5_txChannelEn);
        Output.deviceBoardParam_numVirtualRx        = get_deviceBoardParam_numVirtualRx(Output.deviceBoardParam_numRxAntennas, Output.deviceBoardParam_numTxAntennas);
        Output.deviceBoardParam_txGain              = get_deviceBoardParam_txGain(Input.evm);
        Output.deviceBoardParam_rxGain              = get_deviceBoardParam_rxGain(Input.factoryCalibCfg_RxGain);

        // Information-Only Parameters
        Output.infoOnly_chirpTime                   = get_infoOnly_chirpTime(Output.chirpParam_samplingRate, Output.chirpParam_numSamples);
        Output.infoOnly_maxBeatFrequency            = get_infoOnly_maxBeatFrequency(Output.sdkVersion, Input.device, Output.adcbufCfg_adcOutputFmt, Output.chirpParam_samplingRate, Input.chirpComnCfg_DfeFirSel);
        Output.infoOnly_validSweepBandwidth         = get_infoOnly_validSweepBandwidth(Output.chirpParam_frequencySlope, Output.infoOnly_chirpTime);
        Output.infoOnly_carrierFrequency            = get_infoOnly_carrierFrequency(Output.chirpParam_startFrequency, Output.chirpParam_frequencySlope, Output.chirpParam_adcValidStartTime, Output.infoOnly_validSweepBandwidth);
        Output.infoOnly_chirpRepitionPeriod         = get_infoOnly_chirpRepitionPeriod(Output.sdkVersion, Input.frameCfg5_NumOfBurstsInFrame, Input.frameCfg5_BurstPeriodicity, Output.deviceBoardParam_numTxAntennas, Output.chirpParam_idleTime, Output.chirpParam_rampEndTime);
        Output.infoOnly_nfftRange                   = get_infoOnly_nfftRange(Output.chirpParam_numSamples, Output.adcbufCfg_adcOutputFmt);
        Output.infoOnly_activeChirpingTime          = get_infoOnly_activeChirpingTime(Output.chirpParam_rampEndTime, Input.frameCfg5_NumOfChirpsInBurst, Input.frameCfg5_NumOfBurstsInFrame, Input.frameCfg_numLoops, Output.deviceBoardParam_numTxAntennas, Output.sdkVersion);
        Output.infoOnly_dutyCycle                   = get_infoOnly_dutyCycle(Input.frameCfg_framePeriodicity, Input.frameCfg5_FramePeriodicity, Output.chirpParam_numChirpLoops, Output.chirpParam_idleTime, Output.chirpParam_rampEndTime, Output.deviceBoardParam_numTxAntennas, Output.sdkVersion);
        Output.infoOnly_complianceChirpTime         = get_infoOnly_complianceChirpTime(Input.frameCfg5_NumOfBurstsInFrame, Output.infoOnly_activeChirpingTime, Output.chirpParam_numChirpLoops, Output.chirpParam_idleTime, Output.chirpParam_rampEndTime, Output.deviceBoardParam_numTxAntennas, Input.frameCfg5_BurstPeriodicity, Output.sdkVersion);
        Output.infoOnly_adcSaturation               = get_infoOnly_adcSaturation(Input.factoryCalibCfg_RxGain, Input.factoryCalibCfg_TxBackoff, Input.chirpComnCfg_ChirpRxHpfSel, Input.chripTimingCfg_ChirpRfFreqSlope)

        // Results
        Output.results_maxRange                     = get_results_maxRange(Output.infoOnly_maxBeatFrequency, Output.chirpParam_frequencySlope);
        Output.results_rangeResolution              = get_results_rangeResolution(Output.infoOnly_validSweepBandwidth);
        Output.results_maxVelocity                  = get_results_maxVelocity(Output.infoOnly_carrierFrequency, Output.infoOnly_chirpRepitionPeriod);
        Output.results_velocityResolution           = get_results_velocityResolution(Output.sdkVersion, Input.frameCfg5_NumOfBurstsInFrame, Input.frameCfg5_NumOfChirpsInBurst, Output.results_maxVelocity, Output.chirpParam_numChirpLoops, Output.deviceBoardParam_numTxAntennas);
        Output.results_radarCubeSize                = get_results_radarCubeSize(Output.infoOnly_nfftRange, Output.chirpParam_numChirpLoops, Output.deviceBoardParam_numRxAntennas, Output.deviceBoardParam_numTxAntennas, Output.adcbufCfg_adcOutputFmt);
        Output.results_endFrequency                 = get_results_endFrequency(Output.chirpParam_startFrequency, Output.chirpParam_frequencySlope, Output.chirpParam_rampEndTime);
    };
        
    var updateInput = function(changes) {
        // Grab the self:
        var adv = this;

        // Log the changes that have occurred TO THE INPUT SIDE of the object:
        for (var k in changes) {
            if (changes.hasOwnProperty(k)) {
                adv.Input[k] = changes[k]; 
            }
        }

        // Calculate the outputs:
        updateOutputComputation(adv);
    };
    // ================================ END: UPDATE FUNCTIONS  ============================================================== 
    // ================================ START: GETTER FUNCTIONS  ============================================================
    var get_adcbufCfg_adcOutputFmt = function(adcMode){
        if (adcMode === '') return '';
        else if (adcMode === 'Real') return 1;
        else return 0;
    }
    
    // Chirp Parameters
    var get_chirpParam_startFrequency = function(sdkVersion, profileCfg_startFreq, chripTimingCfg_ChirpRfFreqStart) {
        if (sdkVersion == '0'){
            return profileCfg_startFreq;
        }
        else if (sdkVersion == '1'){
            return chripTimingCfg_ChirpRfFreqStart;
        }
    };
    var get_chirpParam_frequencySlope = function(sdkVersion, profileCfg_freqSlope, chripTimingCfg_ChirpRfFreqSlope) {
        if (sdkVersion == '0'){
            return profileCfg_freqSlope;
        }
        else if (sdkVersion == '1'){
            return chripTimingCfg_ChirpRfFreqSlope;
        }
    };
    var get_chirpParam_samplingRate = function(sdkVersion, profileCfg_digOutSampleRate, chirpComnCfg_DigOutputSampRate) {
        if (sdkVersion == '0'){
            if (profileCfg_digOutSampleRate === '') return '';
            return (profileCfg_digOutSampleRate / 1000);
        }
        else if (sdkVersion == '1'){
            if (chirpComnCfg_DigOutputSampRate === '') return '';
            return (100/chirpComnCfg_DigOutputSampRate);
        }
    };
    var get_chirpParam_numSamples = function(sdkVersion, profileCfg_numAdcSamples, chirpComnCfg_NumOfAdcSamples) {
        if (sdkVersion == '0'){
            return profileCfg_numAdcSamples;
        }
        else if (sdkVersion == '1'){
            return chirpComnCfg_NumOfAdcSamples;
        }
    };
    var get_chirpParam_numChirpLoops = function(sdkVersion, frameCfg_numLoops, frameCfg5_NumOfBurstsInFrame, frameCfg5_NumOfChirpsInBurst) {
        if (sdkVersion == '0'){
            return frameCfg_numLoops;
        }
        else if (sdkVersion == '1'){
            return (frameCfg5_NumOfBurstsInFrame*frameCfg5_NumOfChirpsInBurst);
        }
    };
    var get_chirpParam_idleTime = function(sdkVersion, profileCfg_idleTime, chripTimingCfg_ChirpIdleTime) {
        if (sdkVersion == '0'){
            return profileCfg_idleTime;
        }
        else if (sdkVersion == '1'){
            return chripTimingCfg_ChirpIdleTime;
        }
    };
    var get_chirpParam_adcValidStartTime = function(sdkVersion, profileCfg_adcStartTime, chirpParam_samplingRate, chirpTimingCfg_ChirpAdcSkipSamples) {
        if (sdkVersion == '0'){
            return profileCfg_adcStartTime;
        }
        else if (sdkVersion == '1'){
            return (chirpTimingCfg_ChirpAdcSkipSamples/chirpParam_samplingRate);
        }
    };
    var get_chirpParam_rampEndTime = function(sdkVersion, profileCfg_rampEndTime, chirpComnCfg_ChirpRampEndTime) {
        if (sdkVersion == '0'){
            return profileCfg_rampEndTime;
        }
        else if (sdkVersion == '1'){
            return chirpComnCfg_ChirpRampEndTime;
        }
    };
    var get_chirpParam_framePeriodicity = function(sdkVersion, frameCfg_framePeriodicity, frameCfg5_FramePeriodicity) {
        if (sdkVersion == '0'){
            return frameCfg_framePeriodicity;
        }
        else if (sdkVersion == '1'){
            return frameCfg5_FramePeriodicity;
        }
    };

    // Device & Board Specific Parameters
    var get_deviceBoardParam_numRxAntennas = function(sdkVersion, channelCfg_rxChannelEn, channelCfg5_rxChannelEn) {
        if(sdkVersion == '0'){    
            if (channelCfg_rxChannelEn === '') return '';
            else if (channelCfg_rxChannelEn >= 15) {
                return 4
            }   
            else if (channelCfg_rxChannelEn < 15 && channelCfg_rxChannelEn >= 7) {
                return 3
            }   
            else if (channelCfg_rxChannelEn < 7 && channelCfg_rxChannelEn >= 3) {
                return 2
            }
            else if (channelCfg_rxChannelEn < 3) {
                return 1
            }
        }
        else if (sdkVersion == '1'){
            if (channelCfg5_rxChannelEn === '') return '';
            else if (channelCfg5_rxChannelEn < 15 && channelCfg5_rxChannelEn >= 7) {
                return 3
            }   
            else if (channelCfg5_rxChannelEn < 7 && channelCfg5_rxChannelEn >= 3) {
                return 2
            }
            else if (channelCfg5_rxChannelEn < 3) {
                return 1
            }            
        }
    };
    var get_deviceBoardParam_numTxAntennas = function(sdkVersion, channelCfg_txChannelEn, channelCfg5_txChannelEn) {
        if(sdkVersion == '0'){           
            if (channelCfg_txChannelEn === '') return '';
            else if (channelCfg_txChannelEn < 15 && channelCfg_txChannelEn >= 7) {
                return 3
            }   
            else if (channelCfg_txChannelEn < 7 && channelCfg_txChannelEn >= 3) {
                return 2
            }
            else if (channelCfg_txChannelEn < 3) {
                return 1
            }
        }
        else if (sdkVersion == '1'){
            if (channelCfg5_txChannelEn === '') return '';
            else if (channelCfg5_txChannelEn < 7 && channelCfg5_txChannelEn >= 3) {
                return 2
            }
            else if (channelCfg5_txChannelEn < 3) {
                return 1
            }
        }
    };
    var get_deviceBoardParam_numVirtualRx = function(deviceBoardParam_numRxAntennas, deviceBoardParam_numTxAntennas) {
        if (deviceBoardParam_numRxAntennas === '' || deviceBoardParam_numTxAntennas === '') return '';
        return (deviceBoardParam_numRxAntennas * deviceBoardParam_numTxAntennas);
    };
    var get_deviceBoardParam_txGain = function(evm) {
        if (evm === '') return '';
        return Adv_Globals.GainsByEVM[evm];
    };
    var get_deviceBoardParam_rxGain = function(evm) {
        if (evm === '') return '';
        return Adv_Globals.GainsByEVM[evm];
    };

    // Information-Only Parameters
    var get_infoOnly_chirpTime = function(chirpParam_samplingRate, chirpParam_numSamples) {
        if (chirpParam_samplingRate === '' || chirpParam_numSamples === '') return '';
        return (chirpParam_numSamples / chirpParam_samplingRate);
    };
    
    var get_infoOnly_maxBeatFrequency = function(sdkVersion, device, adcbufCfg_adcOutputFmt, chirpParam_samplingRate, chirpComnCfg_DfeFirSel) {
        // TODO TEST
        var visibilityFactor;
        if (sdkVersion == '1' && chirpComnCfg_DfeFirSel == '0') {
            visibilityFactor = 0.9;
        }
        else {
            visibilityFactor = 0.8;
        }
        if(adcbufCfg_adcOutputFmt == 0){ // complex 1x
            if (chirpParam_samplingRate === '' || device === '') return '';
            return Math.min((chirpParam_samplingRate * visibilityFactor),(Adv_Globals.Platforms[device].maxInterFreq_MHz));
        }
        else if (adcbufCfg_adcOutputFmt == 1){ // real
            if (chirpParam_samplingRate === '' || device === '') return '';
            return Math.min((chirpParam_samplingRate * visibilityFactor)/2,(Adv_Globals.Platforms[device].maxInterFreq_MHz));
        }
    };
    var get_infoOnly_validSweepBandwidth = function(chirpParam_frequencySlope, infoOnly_chirpTime) {
        // =E27*E46
        if (chirpParam_frequencySlope === '' || infoOnly_chirpTime === '') return '';
        return (chirpParam_frequencySlope * infoOnly_chirpTime);
    };
    var get_infoOnly_carrierFrequency = function(chirpParam_startFrequency, chirpParam_frequencySlope, chirpParam_adcValidStartTime, infoOnly_validSweepBandwidth) {
        // =E26+E27*E32/1000+E48/(2*1000)
        if (chirpParam_startFrequency === '' || chirpParam_frequencySlope === '' || chirpParam_adcValidStartTime === '' || infoOnly_validSweepBandwidth === '') return '';
        return (chirpParam_startFrequency + ((chirpParam_frequencySlope * chirpParam_adcValidStartTime) / 1000) + (infoOnly_validSweepBandwidth / (2 * 1000)));
    };
    var get_infoOnly_chirpRepitionPeriod = function(sdkVersion, frameCfg5_NumOfBurstsInFrame, frameCfg5_BurstPeriodicity, deviceBoardParam_numTxAntennas, chirpParam_idleTime, chirpParam_rampEndTime) {
        if (sdkVersion == '0'){
            if (deviceBoardParam_numTxAntennas === '' || chirpParam_idleTime === '' || chirpParam_rampEndTime === '') return '';
            return (deviceBoardParam_numTxAntennas * (chirpParam_idleTime + chirpParam_rampEndTime));
        }
        else if (sdkVersion == '1' && frameCfg5_NumOfBurstsInFrame > 1){
            if (frameCfg5_BurstPeriodicity === '') return '';
           return (frameCfg5_BurstPeriodicity);
        }
        else if (sdkVersion == '1' && frameCfg5_NumOfBurstsInFrame <=1){
            if (deviceBoardParam_numTxAntennas === '' || chirpParam_idleTime === '' || chirpParam_rampEndTime === '') return '';
            return (deviceBoardParam_numTxAntennas * (chirpParam_idleTime + chirpParam_rampEndTime));
        }
    };
    var get_infoOnly_nfftRange = function(chirpParam_numSamples, adcbufCfg_adcOutputFmt) {
        // =POWER(2,CEILING(LOG(E29,2),1))
        if (adcbufCfg_adcOutputFmt == '0') { // Complex
            if (chirpParam_numSamples === '') return '';
            return Math.pow(2, Math.ceil(Math.log2(chirpParam_numSamples)));
        }
        else {
            if (chirpParam_numSamples === '') return '';
            return Math.pow(2, Math.ceil(Math.log2(chirpParam_numSamples))) / 2; // Divided by two for real only, only saving half of the spectrum
        }
    };

    // Results
    var get_results_maxRange = function(infoOnly_maxBeatFrequency, chirpParam_frequencySlope) {
        // (IFmax*c)/(2*S)
        // =(E47*LUT!C4)/(2*Tool!E27*10^6)
        if (infoOnly_maxBeatFrequency === '' || chirpParam_frequencySlope === '') return '';
        return ((infoOnly_maxBeatFrequency * Adv_Globals.Constants.lightSpeed_mps) / (2 * chirpParam_frequencySlope * 1e6));
    };
    var get_results_rangeResolution = function(infoOnly_validSweepBandwidth) {
        // c/(2*B)
        // =LUT!C4/(2*E48*10^6)
        if (infoOnly_validSweepBandwidth === '') return '';
        return (Adv_Globals.Constants.lightSpeed_mps / (2 * infoOnly_validSweepBandwidth * 1e6) );
    };
    var get_results_maxVelocity = function(infoOnly_carrierFrequency, infoOnly_chirpRepitionPeriod) {
        // c/(4*Tr*fc)
        // =LUT!C4/(4*E49*1000*E50)
        if (infoOnly_carrierFrequency === '' || infoOnly_chirpRepitionPeriod === '') return '';
        return (Adv_Globals.Constants.lightSpeed_mps / (4 * infoOnly_carrierFrequency * 1000 * infoOnly_chirpRepitionPeriod));
    };
    var get_results_velocityResolution = function(sdkVersion, frameCfg5_NumOfBurstsInFrame, frameCfg5_NumOfChirpsInBurst, results_maxVelocity, chirpParam_numChirpLoops, deviceBoardParam_numTxAntennas) {
        // 2*vmax/N
        // =2*(E42)/E30
        // TODO TEST

        if (sdkVersion == '0'){
            if (results_maxVelocity === '' || chirpParam_numChirpLoops === '') return '';
            return ((2 * results_maxVelocity) / chirpParam_numChirpLoops);
        }
        else if (sdkVersion == '1'){
            if (results_maxVelocity === '' || frameCfg5_NumOfChirpsInBurst === '') return '';
            if (frameCfg5_NumOfBurstsInFrame > 1) {
                // should be dividing by number of chirps per Tx
                return ((2 * results_maxVelocity) / (frameCfg5_NumOfBurstsInFrame / deviceBoardParam_numTxAntennas));
            }
            else {
                // We do not care about total # of chirps, just # of chirps per Tx
                return ((2 * results_maxVelocity) / chirpParam_numChirpLoops);             
            }
        }
    };
    var get_results_radarCubeSize = function(infoOnly_nfftRange, chirpParam_numChirpLoops, deviceBoardParam_numRxAntennas, deviceBoardParam_numTxAntennas, adcbufCfg_adcOutputFmt) {
        // =E51*E30*E36*E35*4/1024
        if (infoOnly_nfftRange === '' || chirpParam_numChirpLoops === '' || deviceBoardParam_numRxAntennas === '' || deviceBoardParam_numTxAntennas === '') return '';
        return ((4 * infoOnly_nfftRange * chirpParam_numChirpLoops * deviceBoardParam_numRxAntennas * deviceBoardParam_numTxAntennas) / 1024);
    };
    var get_results_endFrequency = function(chirpParam_startFrequency, chirpParam_frequencySlope, chirpParam_rampEndTime) {
        // =E26+(E27*E33/1000)
        if (chirpParam_startFrequency === '' || chirpParam_frequencySlope === '' || chirpParam_rampEndTime === '') return '';
        return (chirpParam_startFrequency + ((chirpParam_frequencySlope * chirpParam_rampEndTime) / 1000));
    };

    // Conditional Cell Gets
    var get_allowed_evms = function(device) {
        if (device in Adv_Globals.Platforms) {
            return Adv_Globals.Platforms[device].evms;
        }
        else {
            return false;
        }
    };
    var get_allowed_adcMode = function(device) {
        if (device in Adv_Globals.Platforms) {
            return Adv_Globals.Platforms[device].adcModes;
        }
        else {
            return false;
        }
    };
    var get_supported_SDK = function(device) {
        if (device in Adv_Globals.Platforms) {
            if(Adv_Globals.Platforms[device].sdkVer === 3) return 0;
            else if(Adv_Globals.Platforms[device].sdkVer === 5) return 1;
            else return -1;
        }
        else return false;
    };
    //time spent bursting within frame
    var get_infoOnly_activeChirpingTime = function(rampEndTime,  frameCfg5_NumOfChirpsInBurst, frameCfg5_NumOfBurstsInFrame, frameCfg_numLoops, deviceBoardParam_numTxAntennas, sdkVersion) {
        if (sdkVersion == '0'){
            return (rampEndTime)*deviceBoardParam_numTxAntennas*frameCfg_numLoops*1E-3;
        }
        else if (sdkVersion == '1'){
            return (rampEndTime)*deviceBoardParam_numTxAntennas*frameCfg5_NumOfChirpsInBurst*frameCfg5_NumOfBurstsInFrame*1E-3;
        }
    };
    // The classic definition of duty cycle: (numChirps * (idleTime_us+rampEndTime_us) / (framePeriodicity_ms * 1000)) * 100%
    var get_infoOnly_dutyCycle = function(framePeriodicitySDK3, framePeriodicitySDK5, numChirpLoops, idleTime, rampEndTime, deviceBoardParam_numTxAntennas,  sdkVersion) {
        if (sdkVersion == '0'){
            return ((deviceBoardParam_numTxAntennas*numChirpLoops) * ((idleTime+rampEndTime) / (framePeriodicitySDK3 * 1000))) * 100;
        }
        else if (sdkVersion == '1'){
            // Since we are assuming BPM for IWRL6432, do not multiply by numTx
            return ((numChirpLoops) * (idleTime+rampEndTime) / (framePeriodicitySDK5 * 1E3)) * 100;
        }
    };
    // A compliance chirp time or "on time" value utilized in FCC regulations. Get the compliance duty cycle by dividing this value by an FCC-regulation assumed 33ms frame period.
    var get_infoOnly_complianceChirpTime = function(frameCfg5_NumOfBurstsInFrame, activeChirpingTime, numChirpLoops, idleTime, rampEndTime, deviceBoardParam_numTxAntennas, BurstPeriodicity, sdkVersion) {
        if (sdkVersion == '0'){
            return (idleTime + rampEndTime)*deviceBoardParam_numTxAntennas*numChirpLoops*1E-3;
        }
        else if (sdkVersion == '1'){
            if (frameCfg5_NumOfBurstsInFrame <= 1){
                return (idleTime + rampEndTime)*deviceBoardParam_numTxAntennas*numChirpLoops*1E-3;
            }
            else if (frameCfg5_NumOfBurstsInFrame >= 2){
                return (BurstPeriodicity * 1E-3 ) * frameCfg5_NumOfBurstsInFrame;
            }
        }
    };
    
    var get_infoOnly_adcSaturation = function(factoryCalibCfg_RxGain, factoryCalibCfg_TxBackoff, chirpComnCfg_ChirpRxHpfSel, chripTimingCfg_ChirpRfFreqSlope) {
    };
    
    // ================================ END: GETTER FUNCTIONS  ==============================================================  
    // ================================ START: OBJECT EXPORTATION  ==========================================================
    mmWaveAdvanced.prototype.init                  = init;
    mmWaveAdvanced.prototype.updateInput           = updateInput;
    mmWaveAdvanced.prototype.get_allowed_evms      = get_allowed_evms;
    mmWaveAdvanced.prototype.get_allowed_adcMode   = get_allowed_adcMode;
    mmWaveAdvanced.prototype.get_supported_SDK     = get_supported_SDK;
    
    // export as AMD/CommonJS module or global variable
    if (typeof define === 'function' && define.amd) define('mmWaveAdvanced', function () { return mmWaveAdvanced; });
    else if (typeof module !== 'undefined') module.exports = mmWaveAdvanced;
    else if (typeof self !== 'undefined') self.mmWaveAdvanced = mmWaveAdvanced;
    else window.mmWaveAdvanced = mmWaveAdvanced;
    // ================================ END: OBJECT EXPORTATION  ============================================================
})();