// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                                                     BASIC TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */

let Basic_Globals = {
    Constants: {
        lightSpeed: 299792458,                  // speed of light m/us   F46
        kB: 1.38064852e-23,               // Bolzmann constant J/K, kgm^2/s^2K  F47
        cube_4pi: Math.pow(4*Math.PI, 3)  // F48
    },
    /**
     * Object of supported devices and their parameters
     */
    Platforms: {
        'xWR6843': {
            evms:       {
                'xWR6843AOP': {
                    tx_gain:    5.2,
                    rx_gain:    5.2,
                    tx_power:   12
                },
                'xWR6843ISK':{
                    tx_gain:    7,
                    rx_gain:    7,
                    tx_power:   12
                },
                'xWR6843ODS':{
                    tx_gain:    5,
                    rx_gain:    5,
                    tx_power:   12
                },
                'xWR6843 Horn Antenna':{
                    tx_gain:    12.8,
                    rx_gain:    12.8,
                    tx_power:   12
                }
            },
            num_tx:     3,
            num_rx:     4,
            freq_range: [[60,64,'60-64']],
            adcModes:   ['Complex 1x','Real'],
            maximum_radar_cube_size:    768,
            maximum_IF_bandwidth:   10,
            maximum_sampling_frequency: 12.5,
        },
        'xWR1843':      {
            evms:       {
                'xWR1843AOP':{
                    tx_gain:    8,
                    rx_gain:    8,
                    tx_power:   16
                },
                'xWR1843BOOST':{
                    tx_gain:    7,
                    rx_gain:    7,
                    tx_power:   12
                }
            },
            num_tx:     3,
            num_rx:     4,
            freq_range: [
                [76,77,'76-77'], 
                [77,81,'77-81']
            ],
            adcModes:   ['Complex 1x','Real'],
            maximum_radar_cube_size:    1024,
            maximum_IF_bandwidth:   10,
            maximum_sampling_frequency: 12.5,
        },
        'xWR1642':      {
            evms:       {
                'xWR1642BOOST':{
                    tx_gain:    10.5,
                    rx_gain:    10.5,
                    tx_power:   12.5
                }
            },
            num_tx:     2,
            num_rx:     4,
            freq_range: [
                [76,77,'76-77'], 
                [77,81,'77-81']
            ],
            adcModes:   ['Complex 1x','Real'],
            maximum_radar_cube_size:    768,
            maximum_IF_bandwidth:   5,
            maximum_sampling_frequency: 6.25,
        },
        'xWRL6432':      {
            evms:       {
                'xWRL6432BOOST':{
                    tx_gain:    5.5,
                    rx_gain:    5.5,
                    tx_power:   11
                },
                'xWRL6432AOP':{
                    tx_gain:    5.5,
                    rx_gain:    5.5,
                    tx_power:   11
                },
            },
            num_tx:     2,
            num_rx:     3,
            freq_range: [
                [57,64,'57-64'],
                [60,64,'60-64']
            ],
            adcModes:   ['Real'],
            maximum_radar_cube_size:    768,
            maximum_IF_bandwidth:   5,
            maximum_sampling_frequency: 6.25,
        },
        'xWRL1432':      {
            evms:       {
                'xWRL1432BOOST':{
                    tx_gain:    5.5,
                    rx_gain:    5.5,
                    tx_power:   11
                }
            },
            num_tx:     2,
            num_rx:     3,
            freq_range: [
                [76,77,'76-77'], 
                [77,81,'77-81']
            ],
            adcModes:   ['Real'],
            maximum_radar_cube_size:    768,
            maximum_IF_bandwidth:   5,
            maximum_sampling_frequency: 6.25,
        },
        'xWR1443':      {
            evms:       {
                'xWR1443BOOST':{
                    tx_gain:    9, 
                    rx_gain:    9,
                    tx_power:   12
                }
            },
            num_tx:     3,
            num_rx:     4,
            freq_range: [
                [76,77,'76-77'], 
                [77,81,'77-81']
            ],
            adcModes:   ['Complex 1x','Real'],
            maximum_radar_cube_size:    384,
            maximum_IF_bandwidth:   15,
            maximum_sampling_frequency: 18.75,
        },
        //automotive only evms
        'AWR1243':      {
            evms:       {
                'AWR1243BOOST':{
                    tx_gain:    7,
                    rx_gain:    7, 
                    tx_power:   12
                }
            },
            num_tx:     3,
            num_rx:     4,
            freq_range: [
                [76,77,'76-77'], 
                [77,81,'77-81'],
                [76,81,'76-81']
            ],
            adcModes:   ['Complex 1x','Real'],
            maximum_radar_cube_size: 1000000,
            maximum_IF_bandwidth: 15,
            maximum_sampling_frequency: 18.75,   
        },
        'AWR2243':      {
            evms:       {
                'AWR2243BOOST':{
                    tx_gain:    7, 
                    rx_gain:    7, 
                    tx_power:   12 
                }
            },
            num_tx:     3,
            num_rx:     4,
            freq_range: [
                [76,77,'76-77'], 
                [77,81,'77-81']
            ],
            adcModes:   ['Complex 1x','Real'],
            maximum_radar_cube_size: 1000000,
            maximum_IF_bandwidth: 20,
            maximum_sampling_frequency: 22.5,
        },
    },
    /**
     * An object skeleton of every input in the BE mmWaveAdvanced Object
     */
    BackendInputs: {
        'device':                   '',
        'evm':                      '',
        'desired_cfg':              '',
        'frequency_range':          '',
        'maximum_detectable_range': '',
        'range_resolution':         '',
        'maximum_velocity_kmph':    '',
        'velocity_resolution_kmph': '',
        'measurement_rate':         '',
        'typical_detected_object':  '',
        'typical_detected_val':     '',
        'detection_loss':           '',
        'system_loss':              '',
        'implementation_margin':    '',
        'detection_SNR':            ''
    },
    /**
     * An object skeleton of every computational intermediate values in the BE mmWaveAdvanced Object
     */
    BackendIntermediates: {
        'maximum_radar_cube_size':'',
        'maximum_IF_bandwidth':'',
        'maximum_sampling_frequency':'',
        'maximum_velocity':'',
        'velocity_resolution':'',
        'valid_sweep_bandwidth':'',
        'idle_time':'',
        'adc_valid_start_time':'',
        'excess_ramping_time':'',
        'periodicity':'',
        'ambient_temperature':'',
        'noise_figure':'',
        'num_virtual_rx':'',
        'non_coherent_combining_loss':'',
        'rcs_value':'',
        'combined_factor_in_dB':'',
        'combined_factor_linear':'',
        'inter_chirp_time':'',
        'aux_comp_coeff_a':'',
        'aux_comp_coeff_b':'',
        'chirp_time_temp':'',
        'ramp_slope_init':'',
        'aux_comp_T1':'',
        'aux_comp_T2':'',
        'carrier_frequency':'',
        'lambda':'',
        'max_chirp_repetition_period':'',
        'max_burst_periodicity':'',
    },
    /**
     * An object skeleton of every output in the BE mmWaveAdvanced Object
     */
    BackendOutputs: {
        'num_rx':                   '',
        'num_tx':                   '',
        'tx_gain':                  '',
        'rx_gain':                  '',
        'maximum_bandwidth':        '',
        'tx_power':                 '',
        
        'max_range_for_typical_detectable_object':'',
        'min_rcs_detectable_at_max_range':'',
        'starting_frequency':'',
        'ramp_slope':'',
        'ramp_slope_parameter':'',
        'sampling_frequency':'',
        'number_of_samples_per_chirp':'',
        'min_num_of_chirp_loops':'', /* change for bursting, number of chirps in a burst */
        'frame_periodicity':'',
        'burst_periodicity':'',
        'bursts_in_frame':'',
        'chirps_in_burst':'',
        'num_bursts':'',
        'idle_time_minimum':'',
        'adc_valid_start_time_2':'',
        'ramp_end_time':'',
        'total_sweep_bandwidth':'',
        'maximum_beat_frequency':'',
        'chirp_time':'',
        'chirp_repetition_period':'',
        'active_frame_time':'',
        'num_range_fft_bins':'',
        'num_doppler_fft_bins':'',
        'range_inter_bin_resolution':'',
        'velocity_inter_bin_resolution':'',
        'radar_cube_size':'',
    },
    /**
     * Mapping of front-end input elements and their back-end variables
     * 
     * [1. html_element_name, 2. mmWaveBasic_var_name, 3. element_type]
     */
    Inputs: [
        ['ti_widget_droplist_device',                       'device',                                   'dropdown'],
        ['ti_widget_droplist_evm',                          'evm',                                      'dropdown'],
        ['ti_widget_droplist_desired_cfg',                  'desired_cfg',                              'dropdown'],
        ['ti_widget_droplist_frequency_range',              'frequency_range',                          'dropdown'],
        ['ti_widget_textbox_ambient_temperature',           'ambient_temperature_degC',                 'float'],
        ['ti_widget_textbox_maximum_detectable_range',      'maximum_detectable_range',                 'float'],
        ['ti_widget_textbox_range_resolution',              'range_resolution',                         'float'],
        ['ti_widget_textbox_maximum_velocity',              'maximum_velocity_kmph',                    'float'],
        ['ti_widget_textbox_velocity_resolution',           'velocity_resolution_kmph',                 'float'],
        ['ti_widget_textbox_measurement_rate',              'measurement_rate',                         'float'],
        ['ti_widget_droplist_typical_detected_object',      'typical_detected_object',                  'dropdown'],
        ['ti_widget_textbox_typical_detected_object',       'typical_detected_val',                     'float'],
        ['ti_widget_textbox_detection_loss',                'detection_loss',                           'float'],
        ['ti_widget_textbox_system_loss',                   'system_loss',                              'float'],
        ['ti_widget_textbox_implementation_margin',         'implementation_margin',                    'float'],
        ['ti_widget_textbox_detection_SNR',                 'detection_SNR',                            'float']
    ],
    /**
     * Default value for each input for each device for each desired configuration default
     * 
     * [1. html_element_name,  2. mmWaveBasic_var_name, 3. {device: {short range default, long range default}}]
     */
    Defaults: [
        ['ti_widget_textbox_ambient_temperature','ambient_temperature_degC', {
                'xWR6843': {'short': 20, 'long': 20},
                'xWRL6432':{'short': 20, 'long': 20},
                'xWRL1432':{'short': 20, 'long': 20},
                'xWR1843': {'short': 20, 'long': 20},
                'xWR1642': {'short': 20, 'long': 20},
                'xWR1443': {'short': 20, 'long': 20},
                'AWR1243': {'short': 20, 'long': 20},
                'AWR2243': {'short': 20, 'long': 20}
            },
        ],
        ['ti_widget_textbox_maximum_detectable_range','maximum_detectable_range', {
                'xWR6843': {'short': 10, 'long': 70},
                'xWRL6432':{'short': 3, 'long': 19},
                'xWRL1432':{'short': 3, 'long': 19},
                'xWR1843': {'short': 10, 'long': 70},
                'xWR1642': {'short': 10, 'long': 65},
                'xWR1443': {'short': 10, 'long': 42},
                'AWR1243': {'short': 10, 'long': 70},
                'AWR2243': {'short': 10, 'long': 70}
            },
        ],
        ['ti_widget_textbox_range_resolution','range_resolution',         {
                'xWR6843': {'short': 5, 'long': 68},
                'xWRL6432':{'short': 5, 'long': 10},
                'xWRL1432':{'short': 15, 'long': 40},
                'xWR1843': {'short': 20, 'long': 68},
                'xWR1642': {'short': 20, 'long': 68},
                'xWR1443': {'short': 20, 'long': 68},
                'AWR1243': {'short': 20, 'long': 68},
                'AWR2243': {'short': 20, 'long': 68}
            },
        ],
        ['ti_widget_textbox_maximum_velocity','maximum_velocity_kmph',    {
                'xWR6843': {'short': 26, 'long': 65},
                'xWRL6432':{'short': 20, 'long': 20},
                'xWRL1432':{'short': 26, 'long': 30},
                'xWR1843': {'short': 26, 'long': 65},
                'xWR1642': {'short': 26, 'long': 65},
                'xWR1443': {'short': 26, 'long': 60},
                'AWR1243': {'short': 26, 'long': 65},
                'AWR2243': {'short': 26, 'long': 65}
            },
        ],
        ['ti_widget_textbox_velocity_resolution','velocity_resolution_kmph', {
                'xWR6843': {'short': 2, 'long': 2},
                'xWRL6432':{'short': 4, 'long': 4},
                'xWRL1432':{'short': 3, 'long': 3},
                'xWR1843': {'short': 2, 'long': 2},
                'xWR1642': {'short': 2, 'long': 2},
                'xWR1443': {'short': 2, 'long': 2},
                'AWR1243': {'short': 2, 'long': 2},
                'AWR2243': {'short': 2, 'long': 2}
            },
        ],
        ['ti_widget_textbox_measurement_rate','measurement_rate',         {
                'xWR6843': {'short': 10, 'long': 25},
                'xWRL6432':{'short': 5, 'long': 15},
                'xWRL1432':{'short': 5, 'long': 15},
                'xWR1843': {'short': 10, 'long': 25},
                'xWR1642': {'short': 10, 'long': 25},
                'xWR1443': {'short': 10, 'long': 25},
                'AWR1243': {'short': 10, 'long': 25},
                'AWR2243': {'short': 10, 'long': 25}
            },
        ],
        ['ti_widget_droplist_typical_detected_object','typical_detected_object',  {
                'xWR6843': {'short': 'Adult', 'long': 'Car'},
                'xWRL6432':{'short': 'Adult', 'long': 'Adult'},
                'xWRL1432':{'short': 'Adult', 'long': 'Adult'},
                'xWR1843': {'short': 'Adult', 'long': 'Car'},
                'xWR1642': {'short': 'Adult', 'long': 'Car'},
                'xWR1443': {'short': 'Adult', 'long': 'Car'},
                'AWR1243': {'short': 'Adult', 'long': 'Car'},
                'AWR2243': {'short': 'Adult', 'long': 'Car'}
            },
        ],
        ['ti_widget_textbox_typical_detected_object','typical_detected_val',     {
                'xWR6843': {'short': 1, 'long': 5},
                'xWRL6432':{'short': 1, 'long': 1},
                'xWRL1432':{'short': 1, 'long': 1},
                'xWR1843': {'short': 1, 'long': 5},
                'xWR1642': {'short': 1, 'long': 5},
                'xWR1443': {'short': 1, 'long': 5},
                'AWR1243': {'short': 1, 'long': 5},
                'AWR2243': {'short': 1, 'long': 5}
            },
        ],
        ['ti_widget_textbox_detection_loss','detection_loss',           {
                'xWR6843': {'short': 1, 'long': 1},
                'xWRL6432':{'short': 1, 'long': 1},
                'xWRL1432':{'short': 1, 'long': 1},
                'xWR1843': {'short': 1, 'long': 1},
                'xWR1642': {'short': 1, 'long': 1},
                'xWR1443': {'short': 1, 'long': 1},
                'AWR1243': {'short': 1, 'long': 1},
                'AWR2243': {'short': 1, 'long': 1}
            },
        ],
        ['ti_widget_textbox_system_loss','system_loss',              {
                'xWR6843': {'short': 1, 'long': 1},
                'xWRL6432':{'short': 1, 'long': 1},
                'xWRL1432':{'short': 1, 'long': 1},
                'xWR1843': {'short': 1, 'long': 1},
                'xWR1642': {'short': 1, 'long': 1},
                'xWR1443': {'short': 1, 'long': 1},
                'AWR1243': {'short': 1, 'long': 1},
                'AWR2243': {'short': 1, 'long': 1}
            },
        ],
        ['ti_widget_textbox_implementation_margin','implementation_margin',    {
                'xWR6843': {'short': 2, 'long': 2},
                'xWRL6432':{'short': 2, 'long': 2},
                'xWRL1432':{'short': 2, 'long': 2},
                'xWR1843': {'short': 2, 'long': 2},
                'xWR1642': {'short': 2, 'long': 2},
                'xWR1443': {'short': 2, 'long': 2},
                'AWR1243': {'short': 2, 'long': 2},
                'AWR2243': {'short': 2, 'long': 2}
            },
        ],
        ['ti_widget_textbox_detection_SNR','detection_SNR',            {
                'xWR6843': {'short': 12, 'long': 12},
                'xWRL6432':{'short': 12, 'long': 12},
                'xWRL1432':{'short': 12, 'long': 12},
                'xWR1843': {'short': 12, 'long': 12},
                'xWR1642': {'short': 12, 'long': 12},
                'xWR1443': {'short': 12, 'long': 12},
                'AWR1243': {'short': 12, 'long': 12},
                'AWR2243': {'short': 12, 'long': 12}
            },
        ]
    ],
    /**
     * Mapping of front-end output elements and their back-end variables
     * 
     * [1. html_element_name, 2. mmWaveBasic_var_name, 3. element_type]
     */
    Outputs: [
        ['ti_widget_textbox_num_rx',                        'num_rx',                                   'int'],
        ['ti_widget_textbox_num_tx',                        'num_tx',                                   'int'],
        ['ti_widget_textbox_tx_gain',                       'tx_gain',                                  'float'],
        ['ti_widget_textbox_rx_gain',                       'rx_gain',                                  'float'],
        ['ti_widget_textbox_max_bandwidth',                 'maximum_bandwidth',                        'int'],
        ['ti_widget_textbox_tx_power',                      'tx_power',                                 'float'],
        ['ti_widget_textbox_max_range_for_det_obj',         'max_range_for_typical_detectable_object',  'float'],
        ['ti_widget_textbox_min_rcs_at_max_range',          'min_rcs_detectable_at_max_range',          'float'],
        ['ti_widget_textbox_start_frequency',               'starting_frequency',                       'int'],
        ['ti_widget_textbox_ramp_slope',                    'ramp_slope',                               'float'],
        ['ti_widget_textbox_ramp_slope_parameter',          'ramp_slope_parameter',                     'float'],
        ['ti_widget_textbox_sampling_frequency',            'sampling_frequency',                       'float'],
        ['ti_widget_textbox_num_samples_per_chirp',         'number_of_samples_per_chirp',              'int'],
        ['ti_widget_textbox_min_num_chirp_loops',           'min_num_of_chirp_loops',                   'int'],
        ['ti_widget_textbox_periodicity',                   'frame_periodicity',                        'float'],
        ['ti_widget_textbox_idle_time_minimum',             'idle_time_minimum',                        'float'],
        ['ti_widget_textbox_adc_valid_start_time',          'adc_valid_start_time_2',                   'float'],
        ['ti_widget_textbox_ramp_end_time',                 'ramp_end_time',                            'float'],
        ['ti_widget_textbox_total_sweep_bandwidth',         'total_sweep_bandwidth',                    'float'],
        ['ti_widget_textbox_maximum_beat_frequency',        'maximum_beat_frequency',                   'float'],
        ['ti_widget_textbox_chirp_time',                    'chirp_time',                               'float'],
        ['ti_widget_textbox_chirp_repetition_period',       'chirp_repetition_period',                  'float'],
        ['ti_widget_textbox_active_frame_time',             'active_frame_time',                        'float'],
        ['ti_widget_textbox_num_range_fft_bins',            'num_range_fft_bins',                       'int'],
        ['ti_widget_textbox_num_doppler_fft_bins',          'num_doppler_fft_bins',                     'int'],
        ['ti_widget_textbox_range_inter_bin_resolution',    'range_inter_bin_resolution',               'float'],
        ['ti_widget_textbox_velocity_inter_bin_resolution', 'velocity_inter_bin_resolution',            'float'],
        ['ti_widget_textbox_radar_cube_size',               'radar_cube_size',                          'float'],
        ['ti_widget_textbox_bursts_in_frame',               'bursts_in_frame',                          'int'],
        ['ti_widget_textbox_chirps_in_burst',               'chirps_in_burst',                          'int'],
        ['ti_widget_textbox_burst_periodicity',             'burst_periodicity',                        'int']
    ],
    /**
     * Storage of OOB Configs per EVM per Device, used in the SAVE CONFIG functionality
     * 
     * {device: {EVM: [OOB CFG LINES]}}
     */
    Configs: {
        'xWR6843':{
            'xWR6843AOP': [
                '% ***************************************************************',
                '% Created for SDK ver:03.06',
                '% Created using SENSING ESTIMATOR BASED OFF OOB DEMO VISUALIZER',
                '% Frequency:60',
                '% Platform:xWR68xx_AOP',
                '% Scene Classifier:best_range_res',
                '% Azimuth Resolution(deg):30 + 30',
                '% Range Resolution(m):0.044',
                '% Maximum unambiguous Range(m):9.02',
                '% Maximum Radial Velocity(m/s):1',
                '% Radial velocity resolution(m/s):0.13',
                '% Frame Duration(msec):100',
                '% RF calibration data:None',
                '% Range Detection Threshold (dB):15',
                '% Doppler Detection Threshold (dB):15',
                '% Range Peak Grouping:enabled',
                '% Doppler Peak Grouping:enabled',
                '% Static clutter removal:disabled',
                '% Angle of Arrival FoV: Full FoV',
                '% Range FoV: Full FoV',
                '% Doppler FoV: Full FoV',
                '% ***************************************************************',
                'sensorStop',
                'flushCfg',
                'dfeDataOutputMode 1',
                'channelCfg 15 7 0',
                'adcCfg 2 1',
                'adcbufCfg -1 0 1 1 1',
                'profileCfg 0 60 359 7 57.14 0 0 70 1 256 5209 0 0 158',
                'chirpCfg 0 0 0 0 0 0 0 1',
                'chirpCfg 1 1 0 0 0 0 0 2',
                'chirpCfg 2 2 0 0 0 0 0 4',
                'frameCfg 0 2 16 0 100 1 0',
                'lowPower 0 0',
                'guiMonitor -1 1 1 0 0 0 1',
                'cfarCfg -1 0 2 8 4 3 0 15 1',
                'cfarCfg -1 1 0 4 2 3 1 15 1',
                'multiObjBeamForming -1 1 0.5',
                'clutterRemoval -1 0',
                'calibDcRangeSig -1 0 -5 8 256',
                'extendedMaxVelocity -1 0',
                'lvdsStreamCfg -1 0 0 0',
                'compRangeBiasAndRxChanPhase 0.0 1 0 -1 0 1 0 -1 0 1 0 -1 0 1 0 -1 0 1 0 -1 0 1 0 -1 0',
                'measureRangeBiasAndRxChanPhase 0 1.5 0.2',
                'CQRxSatMonitor 0 3 5 121 0',
                'CQSigImgMonitor 0 127 4',
                'analogMonitor 0 0',
                'aoaFovCfg -1 -90 90 -90 90',
                'cfarFovCfg -1 0 0 8.92',
                'cfarFovCfg -1 1 -1 1.00',
                'calibData 0 0 0',
                'sensorStart',
            ],
            'xWR6843ISK':[
                '% ***************************************************************',
                '% Created for SDK ver:03.06',
                '% Created using SENSING ESTIMATOR BASED OFF OOB DEMO VISUALIZER',
                '% Frequency:60',
                '% Platform:xWR68xx',
                '% Scene Classifier:best_range_res',
                '% Azimuth Resolution(deg):15 + Elevation',
                '% Range Resolution(m):0.044',
                '% Maximum unambiguous Range(m):9.02',
                '% Maximum Radial Velocity(m/s):1',
                '% Radial velocity resolution(m/s):0.13',
                '% Frame Duration(msec):100',
                '% RF calibration data:None',
                '% Range Detection Threshold (dB):15',
                '% Doppler Detection Threshold (dB):15',
                '% Range Peak Grouping:enabled',
                '% Doppler Peak Grouping:enabled',
                '% Static clutter removal:disabled',
                '% Angle of Arrival FoV: Full FoV',
                '% Range FoV: Full FoV',
                '% Doppler FoV: Full FoV',
                '% ***************************************************************',
                'sensorStop',
                'flushCfg',
                'dfeDataOutputMode 1',
                'channelCfg 15 7 0',
                'adcCfg 2 1',
                'adcbufCfg -1 0 1 1 1',
                'profileCfg 0 60 359 7 57.14 0 0 70 1 256 5209 0 0 158',
                'chirpCfg 0 0 0 0 0 0 0 1',
                'chirpCfg 1 1 0 0 0 0 0 4',
                'chirpCfg 2 2 0 0 0 0 0 2',
                'frameCfg 0 2 16 0 100 1 0',
                'lowPower 0 0',
                'guiMonitor -1 1 1 0 0 0 1',
                'cfarCfg -1 0 2 8 4 3 0 15 1',
                'cfarCfg -1 1 0 4 2 3 1 15 1',
                'multiObjBeamForming -1 1 0.5',
                'clutterRemoval -1 0',
                'calibDcRangeSig -1 0 -5 8 256',
                'extendedMaxVelocity -1 0',
                'bpmCfg -1 0 0 1',
                'lvdsStreamCfg -1 0 0 0',
                'compRangeBiasAndRxChanPhase 0.0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0',
                'measureRangeBiasAndRxChanPhase 0 1.5 0.2',
                'CQRxSatMonitor 0 3 5 121 0',
                'CQSigImgMonitor 0 127 4',
                'analogMonitor 0 0',
                'aoaFovCfg -1 -90 90 -90 90',
                'cfarFovCfg -1 0 0 8.92',
                'cfarFovCfg -1 1 -1 1.00',
                'calibData 0 0 0',
                'sensorStart',
            ],
            'xWR6843ODS':[
                '% ***************************************************************',
                '% Created for SDK ver:03.06',
                '% Created using SENSING ESTIMATOR BASED OFF OOB DEMO VISUALIZER',
                '% Frequency:60',
                '% Platform:xWR68xx',
                '% Scene Classifier:best_range_res',
                '% Azimuth Resolution(deg):15 + Elevation',
                '% Range Resolution(m):0.044',
                '% Maximum unambiguous Range(m):9.02',
                '% Maximum Radial Velocity(m/s):1',
                '% Radial velocity resolution(m/s):0.13',
                '% Frame Duration(msec):100',
                '% RF calibration data:None',
                '% Range Detection Threshold (dB):15',
                '% Doppler Detection Threshold (dB):15',
                '% Range Peak Grouping:enabled',
                '% Doppler Peak Grouping:enabled',
                '% Static clutter removal:disabled',
                '% Angle of Arrival FoV: Full FoV',
                '% Range FoV: Full FoV',
                '% Doppler FoV: Full FoV',
                '% ***************************************************************',
                'sensorStop',
                'flushCfg',
                'dfeDataOutputMode 1',
                'channelCfg 15 7 0',
                'adcCfg 2 1',
                'adcbufCfg -1 0 1 1 1',
                'profileCfg 0 60 359 7 57.14 0 0 70 1 256 5209 0 0 158',
                'chirpCfg 0 0 0 0 0 0 0 1',
                'chirpCfg 1 1 0 0 0 0 0 4',
                'chirpCfg 2 2 0 0 0 0 0 2',
                'frameCfg 0 2 16 0 100 1 0',
                'lowPower 0 0',
                'guiMonitor -1 1 1 0 0 0 1',
                'cfarCfg -1 0 2 8 4 3 0 15 1',
                'cfarCfg -1 1 0 4 2 3 1 15 1',
                'multiObjBeamForming -1 1 0.5',
                'clutterRemoval -1 0',
                'calibDcRangeSig -1 0 -5 8 256',
                'extendedMaxVelocity -1 0',
                'bpmCfg -1 0 0 1',
                'lvdsStreamCfg -1 0 0 0',
                'compRangeBiasAndRxChanPhase 0.0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0',
                'measureRangeBiasAndRxChanPhase 0 1.5 0.2',
                'CQRxSatMonitor 0 3 5 121 0',
                'CQSigImgMonitor 0 127 4',
                'analogMonitor 0 0',
                'aoaFovCfg -1 -90 90 -90 90',
                'cfarFovCfg -1 0 0 8.92',
                'cfarFovCfg -1 1 -1 1.00',
                'calibData 0 0 0',
                'sensorStart',
            ],
            'xWR6843 Horn Antenna':[
                '% ***************************************************************',
                '% Created for SDK ver:03.06',
                '% Created using SENSING ESTIMATOR BASED OFF OOB DEMO VISUALIZER',
                '% Frequency:60',
                '% Platform:xWR68xx',
                '% Scene Classifier:best_range_res',
                '% Azimuth Resolution(deg):15 + Elevation',
                '% Range Resolution(m):0.044',
                '% Maximum unambiguous Range(m):9.02',
                '% Maximum Radial Velocity(m/s):1',
                '% Radial velocity resolution(m/s):0.13',
                '% Frame Duration(msec):100',
                '% RF calibration data:None',
                '% Range Detection Threshold (dB):15',
                '% Doppler Detection Threshold (dB):15',
                '% Range Peak Grouping:enabled',
                '% Doppler Peak Grouping:enabled',
                '% Static clutter removal:disabled',
                '% Angle of Arrival FoV: Full FoV',
                '% Range FoV: Full FoV',
                '% Doppler FoV: Full FoV',
                '% ***************************************************************',
                'sensorStop',
                'flushCfg',
                'dfeDataOutputMode 1',
                'channelCfg 15 7 0',
                'adcCfg 2 1',
                'adcbufCfg -1 0 1 1 1',
                'profileCfg 0 60 359 7 57.14 0 0 70 1 256 5209 0 0 158',
                'chirpCfg 0 0 0 0 0 0 0 1',
                'chirpCfg 1 1 0 0 0 0 0 4',
                'chirpCfg 2 2 0 0 0 0 0 2',
                'frameCfg 0 2 16 0 100 1 0',
                'lowPower 0 0',
                'guiMonitor -1 1 1 0 0 0 1',
                'cfarCfg -1 0 2 8 4 3 0 15 1',
                'cfarCfg -1 1 0 4 2 3 1 15 1',
                'multiObjBeamForming -1 1 0.5',
                'clutterRemoval -1 0',
                'calibDcRangeSig -1 0 -5 8 256',
                'extendedMaxVelocity -1 0',
                'bpmCfg -1 0 0 1',
                'lvdsStreamCfg -1 0 0 0',
                'compRangeBiasAndRxChanPhase 0.0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0',
                'measureRangeBiasAndRxChanPhase 0 1.5 0.2',
                'CQRxSatMonitor 0 3 5 121 0',
                'CQSigImgMonitor 0 127 4',
                'analogMonitor 0 0',
                'aoaFovCfg -1 -90 90 -90 90',
                'cfarFovCfg -1 0 0 8.92',
                'cfarFovCfg -1 1 -1 1.00',
                'calibData 0 0 0',
                'sensorStart',
            ]
        },
        'xWRL6432':{
            'xWRL6432BOOST':[
                '% ***************************************************************',
                '% MotionDetect:  Chirp configuration and Processing chain are', 
                '% designed to detect moving objects, including estimation of', 
                '% range, velocity and angle of the objects.  It is typically', 
                '% useful for detecting, localizing and tracking objects in', 
                '% indoor or outdoor settings.',
                '% ***************************************************************',
                'channelCfg 7 3 0',
                'chirpComnCfg 8 0 0 256 4 28 0',
                'chirpTimingCfg 6 63 0 75 60',
                'frameCfg 2 0 200 64 250 0',
                'guiMonitor 2 1 0 0 0 1 0 0 0 0 0',
                'sigProcChainCfg 32 2 1 0 4 4 0 15', 
                'cfarCfg 2 8 4 3 0 12.0 0 0.5 0 1 1 1',
                'aoaFovCfg -60 60 -40 40',
                'rangeSelCfg 0.1 12.0',
                'clutterRemoval 1',
                'compRangeBiasAndRxChanPhase 0.0 1.00000 0.00000 -1.00000 0.00000 1.00000 0.00000 -1.00000 0.00000 1.00000 0.00000 -1.00000 0.00000',
                'adcDataSource 0 C:/ti/mmwave_lp_sdk/examples/datapath/common/testBench/major_motion/adc_data_0001_CtestAdc6Ant.bin',
                'adcLogging 0',
                'lowPowerCfg 1',
                'factoryCalibCfg 1 0 40 0 0x1ff000',
                'mpdBoundaryBox 1 0 1.48 0 1.95 0 3',
                'mpdBoundaryBox 2 0 1.48 1.95 3.9 0 3',
                'mpdBoundaryBox 3 -1.48 0 0 1.95 0 3',
                'mpdBoundaryBox 4 -1.48 0 1.95 3.9 0 3',
                'sensorPosition 0 0 1.44 0 0',
                'minorStateCfg 5 4 40 8 4 30 8 8',
                'majorStateCfg 4 2 30 10 8 80 4 4',
                'clusterCfg 1 0.5 2',
                'baudRate 1250000',
                'sensorStart 0 0 0 0',
            ],
            'xWRL6432AOP':[
                '% ***************************************************************',
                '% MotionDetect:  Chirp configuration and Processing chain are', 
                '% designed to detect moving objects, including estimation of', 
                '% range, velocity and angle of the objects.  It is typically', 
                '% useful for detecting, localizing and tracking objects in', 
                '% indoor or outdoor settings.',
                '% ***************************************************************',
                'channelCfg 7 3 0',
                'chirpComnCfg 8 0 0 256 1 28 2',
                'chirpTimingCfg 6 63 0 75 60',
                'frameCfg 2 0 200 64 250 0',
                'guiMonitor 2 1 0 0 0 1 0 0 0 0 0',
                'sigProcChainCfg 32 2 1 0 4 4 0 15', 
                'cfarCfg 2 8 4 3 0 12.0 0 0.8 0 1 1 1',
                'aoaFovCfg -60 60 -40 40',
                'rangeSelCfg 0.1 12.0',
                'clutterRemoval 1',
                'antGeometryCfg 1 1 1 0 0 1 1 3 1 2 0 3 2.5 2.5',
                'compRangeBiasAndRxChanPhase 0.0 1.00000 0.00000 1.00000 0.00000 1.00000 0.00000 1.00000 0.00000 1.00000 0.00000 1.00000 0.00000',
                'adcDataSource 0 C:/ti/mmwave_lp_sdk/examples/datapath/common/testBench/major_motion/adc_data_0001_CtestAdc6Ant.bin',
                'adcLogging 0',
                'lowPowerCfg 1',
                'factoryCalibCfg 1 0 30 0 0x1ff000',
                'mpdBoundaryBox 1 0 1.48 0 1.95 0 3',
                'mpdBoundaryBox 2 0 1.48 1.95 3.9 0 3',
                'mpdBoundaryBox 3 -1.48 0 0 1.95 0 3',
                'mpdBoundaryBox 4 -1.48 0 1.95 3.9 0 3',
                'sensorPosition 0 0 1.44 0 0',
                'minorStateCfg 5 4 40 8 4 30 8 8',
                'majorStateCfg 4 2 30 10 8 80 4 4',
                'clusterCfg 1 0.5 2',
                'baudRate 1250000',
                'sensorStart 0 0 0 0',
            ]
        },
        'xWRL1432':{
            'xWRL1432BOOST':[
                '% ***************************************************************',
                '% MotionDetect:  Chirp configuration and Processing chain are ',
                '% designed to detect moving objects, including estimation of ',
                '% range, velocity and angle of the objects.  It is typically ',
                '% useful for detecting, localizing and tracking objects in ',
                '% indoor or outdoor settings.',
                '% ***************************************************************',
                'channelCfg 7 3 0',
                'chirpComnCfg 8 0 0 256 4 28 0',
                'chirpTimingCfg 6 63 0 75 77',
                'frameCfg 2 0 200 64 250 0',
                'guiMonitor 2 1 0 0 0 1 0 0 0 0 0',
                'sigProcChainCfg 32 2 1 0 4 4 0 15' ,
                'cfarCfg 2 8 4 3 0 12.0 0 0.5 0 1 1 1',
                'aoaFovCfg -60 60 -40 40',
                'rangeSelCfg 0.1 12.0',
                'clutterRemoval 1',
                'compRangeBiasAndRxChanPhase 0.0 1.00000 0.00000 -1.00000 0.00000 1.00000 0.00000 -1.00000 0.00000 1.00000 0.00000 -1.00000 0.00000',
                'adcDataSource 0 C:/ti/mmwave_lp_sdk/examples/datapath/common/testBench/major_motion/adc_data_0001_CtestAdc6Ant.bin',
                'adcLogging 0',
                'lowPowerCfg 1',
                'factoryCalibCfg 1 0 40 0 0x1ff000',
                'mpdBoundaryBox 1 0 1.48 0 1.95 0 3',
                'mpdBoundaryBox 2 0 1.48 1.95 3.9 0 3',
                'mpdBoundaryBox 3 -1.48 0 0 1.95 0 3',
                'mpdBoundaryBox 4 -1.48 0 1.95 3.9 0 3',
                'sensorPosition 0 0 1.44 0 0',
                'minorStateCfg 5 4 40 8 4 30 8 8',
                'majorStateCfg 4 2 30 10 8 80 4 4',
                'clusterCfg 1 0.5 2',
                'baudRate 1250000',
                'sensorStart 0 0 0 0',
            ]
        },
       'xWR1843':{
            'xWR1843AOP':[
                '% ***************************************************************',
                '% Created for SDK ver:03.06',
                '% Created using SENSING ESTIMATOR BASED OFF OOB DEMO VISUALIZER',
                '% Frequency:77',
                '% Platform:xWR18xx',
                '% Scene Classifier:best_range_res',
                '% Azimuth Resolution(deg):15 + Elevation',
                '% Range Resolution(m):0.044',
                '% Maximum unambiguous Range(m):9.02',
                '% Maximum Radial Velocity(m/s):1',
                '% Radial velocity resolution(m/s):0.13',
                '% Frame Duration(msec):100',
                '% RF calibration data:None',
                '% Range Detection Threshold (dB):15',
                '% Doppler Detection Threshold (dB):15',
                '% Range Peak Grouping:enabled',
                '% Doppler Peak Grouping:enabled',
                '% Static clutter removal:disabled',
                '% Angle of Arrival FoV: Full FoV',
                '% Range FoV: Full FoV',
                '% Doppler FoV: Full FoV',
                '% ***************************************************************',
                'sensorStop',
                'flushCfg',
                'dfeDataOutputMode 1',
                'channelCfg 15 7 0',
                'adcCfg 2 1',
                'adcbufCfg -1 0 1 1 1',
                'profileCfg 0 77 267 7 57.14 0 0 70 1 256 5209 0 0 30',
                'chirpCfg 0 0 0 0 0 0 0 1',
                'chirpCfg 1 1 0 0 0 0 0 4',
                'chirpCfg 2 2 0 0 0 0 0 2',
                'frameCfg 0 2 16 0 100 1 0',
                'lowPower 0 0',
                'guiMonitor -1 1 1 0 0 0 1',
                'cfarCfg -1 0 2 8 4 3 0 15 1',
                'cfarCfg -1 1 0 4 2 3 1 15 1',
                'multiObjBeamForming -1 1 0.5',
                'clutterRemoval -1 0',
                'calibDcRangeSig -1 0 -5 8 256',
                'extendedMaxVelocity -1 0',
                'lvdsStreamCfg -1 0 0 0',
                'compRangeBiasAndRxChanPhase 0.0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0',
                'measureRangeBiasAndRxChanPhase 0 1.5 0.2',
                'CQRxSatMonitor 0 3 5 121 0',
                'CQSigImgMonitor 0 127 4',
                'analogMonitor 0 0',
                'aoaFovCfg -1 -90 90 -90 90',
                'cfarFovCfg -1 0 0 8.92',
                'cfarFovCfg -1 1 -1 1.00',
                'calibData 0 0 0',
                'sensorStart',
            ],
            'xWR1843BOOST':[
                '% ***************************************************************',
                '% Created for SDK ver:03.06',
                '% Created using SENSING ESTIMATOR BASED OFF OOB DEMO VISUALIZER',
                '% Frequency:77',
                '% Platform:xWR18xx',
                '% Scene Classifier:best_range_res',
                '% Azimuth Resolution(deg):15 + Elevation',
                '% Range Resolution(m):0.044',
                '% Maximum unambiguous Range(m):9.02',
                '% Maximum Radial Velocity(m/s):1',
                '% Radial velocity resolution(m/s):0.13',
                '% Frame Duration(msec):100',
                '% RF calibration data:None',
                '% Range Detection Threshold (dB):15',
                '% Doppler Detection Threshold (dB):15',
                '% Range Peak Grouping:enabled',
                '% Doppler Peak Grouping:enabled',
                '% Static clutter removal:disabled',
                '% Angle of Arrival FoV: Full FoV',
                '% Range FoV: Full FoV',
                '% Doppler FoV: Full FoV',
                '% ***************************************************************',
                'sensorStop',
                'flushCfg',
                'dfeDataOutputMode 1',
                'channelCfg 15 7 0',
                'adcCfg 2 1',
                'adcbufCfg -1 0 1 1 1',
                'profileCfg 0 77 267 7 57.14 0 0 70 1 256 5209 0 0 30',
                'chirpCfg 0 0 0 0 0 0 0 1',
                'chirpCfg 1 1 0 0 0 0 0 4',
                'chirpCfg 2 2 0 0 0 0 0 2',
                'frameCfg 0 2 16 0 100 1 0',
                'lowPower 0 0',
                'guiMonitor -1 1 1 0 0 0 1',
                'cfarCfg -1 0 2 8 4 3 0 15 1',
                'cfarCfg -1 1 0 4 2 3 1 15 1',
                'multiObjBeamForming -1 1 0.5',
                'clutterRemoval -1 0',
                'calibDcRangeSig -1 0 -5 8 256',
                'extendedMaxVelocity -1 0',
                'lvdsStreamCfg -1 0 0 0',
                'compRangeBiasAndRxChanPhase 0.0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0',
                'measureRangeBiasAndRxChanPhase 0 1.5 0.2',
                'CQRxSatMonitor 0 3 5 121 0',
                'CQSigImgMonitor 0 127 4',
                'analogMonitor 0 0',
                'aoaFovCfg -1 -90 90 -90 90',
                'cfarFovCfg -1 0 0 8.92',
                'cfarFovCfg -1 1 -1 1.00',
                'calibData 0 0 0',
                'sensorStart',
            ]
       },
       'xWR1642':{
            'xWR1642BOOST':[
                '% ***************************************************************',
                '% Created for SDK ver:03.06',
                '% Created using SENSING ESTIMATOR BASED OFF OOB DEMO VISUALIZER',
                '% Frequency:77',
                '% Platform:xWR16xx',
                '% Scene Classifier:best_range_res',
                '% Azimuth Resolution(deg):15',
                '% Range Resolution(m):0.044',
                '% Maximum unambiguous Range(m):9.02',
                '% Maximum Radial Velocity(m/s):1',
                '% Radial velocity resolution(m/s):0.13',
                '% Frame Duration(msec):100',
                '% RF calibration data:None',
                '% Range Detection Threshold (dB):15',
                '% Doppler Detection Threshold (dB):15',
                '% Range Peak Grouping:enabled',
                '% Doppler Peak Grouping:enabled',
                '% Static clutter removal:disabled',
                '% Angle of Arrival FoV: Full FoV',
                '% Range FoV: Full FoV',
                '% Doppler FoV: Full FoV',
                '% ***************************************************************',
                'sensorStop',
                'flushCfg',
                'dfeDataOutputMode 1',
                'channelCfg 15 3 0',
                'adcCfg 2 1',
                'adcbufCfg -1 0 1 1 0',
                'profileCfg 0 77 429 7 57.14 0 0 70 1 256 5209 0 0 30',
                'chirpCfg 0 0 0 0 0 0 0 1',
                'chirpCfg 1 1 0 0 0 0 0 2',
                'frameCfg 0 1 16 0 100 1 0',
                'lowPower 0 1',
                'guiMonitor -1 1 1 0 0 0 1',
                'cfarCfg -1 0 2 8 4 3 0 15 1',
                'cfarCfg -1 1 0 4 2 3 1 15 1',
                'multiObjBeamForming -1 1 0.5',
                'clutterRemoval -1 0',
                'calibDcRangeSig -1 0 -5 8 256',
                'extendedMaxVelocity -1 0',
                'bpmCfg -1 0 0 1',
                'lvdsStreamCfg -1 0 0 0',
                'compRangeBiasAndRxChanPhase 0.0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0',
                'measureRangeBiasAndRxChanPhase 0 1.5 0.2',
                'CQRxSatMonitor 0 3 5 121 0',
                'CQSigImgMonitor 0 127 4',
                'analogMonitor 0 0',
                'aoaFovCfg -1 -90 90 -90 90',
                'cfarFovCfg -1 0 0 8.92',
                'cfarFovCfg -1 1 -1 1.00',
                'calibData 0 0 0',
                'sensorStart',
            ]
       },
       'xWR1443':{
            'xWR1443BOOST':[
                '% ***************************************************************',
                '% Created for SDK ver:02.01',
                '% Created using SENSING ESTIMATOR BASED OFF OOB DEMO VISUALIZER',
                '% Frequency:77',
                '% Platform:xWR14xx',
                '% Scene Classifier:best_range_res',
                '% Azimuth Resolution(deg):15 + Elevation',
                '% Range Resolution(m):0.044',
                '% Maximum unambiguous Range(m):9.01',
                '% Maximum Radial Velocity(m/s):1',
                '% Radial velocity resolution(m/s):0.13',
                '% Frame Duration(msec):100',
                '% Range Detection Threshold (dB):15',
                '% Range Peak Grouping:enabled',
                '% Doppler Peak Grouping:enabled',
                '% Static clutter removal:disabled',
                '% ***************************************************************',
                'sensorStop',
                'flushCfg',
                'dfeDataOutputMode 1',
                'channelCfg 15 7 0',
                'adcCfg 2 1',
                'adcbufCfg 0 1 0 1',
                'profileCfg 0 77 267 7 57.14 0 0 70 1 240 4884 0 0 30',
                'chirpCfg 0 0 0 0 0 0 0 1',
                'chirpCfg 1 1 0 0 0 0 0 4',
                'chirpCfg 2 2 0 0 0 0 0 2',
                'frameCfg 0 2 16 0 100 1 0',
                'lowPower 0 1',
                'guiMonitor 1 1 0 0 0 1',
                'cfarCfg 0 2 8 4 3 0 1280',
                'peakGrouping 1 1 1 1 229',
                'multiObjBeamForming 1 0.5',
                'clutterRemoval 0',
                'calibDcRangeSig 0 -5 8 256',
                'compRangeBiasAndRxChanPhase 0.0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1 0',
                'measureRangeBiasAndRxChanPhase 0 1.5 0.2',
                'CQRxSatMonitor 0 3 5 123 0',
                'CQSigImgMonitor 0 119 4',
                'analogMonitor 1 1',
                'sensorStart',
            ]
        }
    },
    /**
     * Buffer to hold the CMD lines for the basic tool
     */
    CmdLines: [],
};