// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                            POWER TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */


(function () {
    // ================================ START: CONSTRUCTORS  ====================================================    
    
    // Constructor call for mmWavePower Obj
    function mmWavePower() {
        if (!(this instanceof mmWavePower))
            return new mmWavePower();
        this.init();
    }
    
    // Init call used in constructor for the mmWavePower Obj
    function init() {
        this.Input = {
            'device':                   '',
            'evm':                      '',
            'mode':                     '',
            'io':                       '',
            'sili':                 '',
            'proc_time':                '',
            'frequency_range':          '',
            'num_rx':                   '',
            'num_tx':                   '',
            'rampendtime':              '',
            'chirpidle':                '',
            'numbursts':                '',
            'frametime':                '',
            'numchirps':                '',
            'chirps_accum':             '',
            'burst_periodicity':        ''
        };
        this.Intermediate = {
            'wakeup_time':              '',
            'power_calc_version':       '',
            'chirps_per_burst':         '',
            'active_time_per_burst':    '',
            'interburst_time_per_burst':'',
            'interburst_trans_time':    '',
            'interframe_trans_time':    '',
            'interframe_trans_power':   '',
            'sleep_transition_time':    '',
            'power_idle_transition':    '',
            'power_sleep_transition':   '',
            'tx_power':                 '',
            'active_time_seconds':      '',
            'interburst_time_seconds':  '',
            'interframe_time_seconds':  ''
        };
        this.Output = {
            'active_time':              '',
            'interburst_time':          '',
            'proc_time':                '',
            'interframe_time':          '',
            'active_usage':             '',
            'interburst_usage':         '',
            'proc_usage':               '',
            'interframe_usage':         '',
            'avg_power':                ''
        };
       
    }
    // ================================ END: CONSTRUCTORS  ======================================================

    // ================================ START: GETTER FUNCTIONS  ============================================================    
    // Getter functions for the Device and EVM Specific Parameters
    var get_allowed_evms = function(device) {
        return Object.keys(Power_Globals.Platforms[device].evms);
    };
    var get_chirps_per_burst = function(numchirps, chirps_accum){
        if(chirps_accum>0){
            return numchirps*chirps_accum;
        }
        else{
            return numchirps;
        }
    };
    var get_allowed_silicon_rev = function(device, evm) {
        if (evm == 'IWRL6432AOP'){
            return ["ES2.0"];    
        }
        else{
            return Object.keys(Power_Globals.Platforms[device].silicon_rev);    
        }
    };
    
    var get_active_time_per_burst = function(rampendtime, chirpidle, chirps_per_burst) {
        return ((rampendtime*1e-6)+(chirpidle*1e-6))*parseInt(chirps_per_burst);
	};
    var get_total_active_time = function(active_time_per_burst, numbursts){
        return active_time_per_burst*numbursts;
    };
    
    var get_interburst_time_per_burst = function(burst_periodicity, active_time_per_burst){
        var time_per_burst = (burst_periodicity*1e-6)-active_time_per_burst;
        if(time_per_burst<=0){
            return 0;
        }
        else{
            return time_per_burst;
        }
    };
    var get_interburst_time = function(interburst_time_per_burst, numbursts){
        return interburst_time_per_burst*(numbursts-1);
    }

    var get_interframe_time = function(frametime, proc_time, burst_periodicity, numbursts, active_time_per_burst){
        var interframe_time = (frametime*1e-3) - (proc_time*1e-3) - ((burst_periodicity*1e-6*(numbursts-1))+active_time_per_burst);
        if(interframe_time <=0){
            return 0;
        }
        else{
            return interframe_time
        }
    };
    var get_active_usage = function(device, num_rx, num_tx, sili, mode, io){
        if(mode==="power"){
            return Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.activeTXRX[num_tx-1][num_rx-1];
        }
        else{
            return Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.activeTXRX[num_tx-1][num_rx-1];
        }
    };
    var get_idle_usage = function(device, idle_time, active_usage, sili, mode, io){
        
        if(mode==="power"){
            //check if interburst period is long enough to enter deep sleep mode
            if(idle_time>=Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.deepSleepTime){
                return {idle_usage:Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.deepSleep[io], idle_trans_time: Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.deepSleepTime, idle_trans_power: Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.deepSleepTransitionPower};
            }
            else if(idle_time>=Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interframeIdleTime && idle_time<Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.deepSleepTime){
                return {idle_usage:Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.interframeIdle, idle_trans_time: Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interframeIdleTime, idle_trans_power: Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.interframeIdleTransitionPower};
            }
            else if(idle_time>=Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interburstIdleTime && idle_time<Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interframeIdleTime){
                return {idle_usage:Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.interburstIdle, idle_trans_time: Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interburstIdleTime, idle_trans_power: Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.interburstIdleTransitionPower};
            }
            else if(idle_time>=Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interchirpIdleTime && idle_time<Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interburstIdleTime){
                return {idle_usage:Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.interchirpIdle, idle_trans_time: Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interchirpIdleTime, idle_trans_power: Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.interchirpIdleTransitionPower};
            }
            //if not enough time to enter any idle mode, then stay in active mode
            else{
                return {idle_usage:active_usage, idle_trans_time: 0, idle_trans_power:0};
            }
        }
        else{
            //check if interburst period is long enough to enter deep sleep mode
            if(idle_time>=Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.deepSleepTime){
                return {idle_usage:Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.deepSleep[io], idle_trans_time: Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.deepSleepTime, idle_trans_power: Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.deepSleepTransitionPower};
            }
            else if(idle_time>=Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interframeIdleTime && idle_time<Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.deepSleepTime){
                return {idle_usage:Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.interframeIdle, idle_trans_time: Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interframeIdleTime, idle_trans_power: Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.interframeIdleTransitionPower};
            }
            else if(idle_time>=Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interburstIdleTime && idle_time<Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interframeIdleTime){
                return {idle_usage:Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.interburstIdle, idle_trans_time: Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interburstIdleTime, idle_trans_power: Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.interburstIdleTransitionPower};
            }
            else if(idle_time>=Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interchirpIdleTime && idle_time<Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interburstIdleTime){
                return {idle_usage:Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.interchirpIdle, idle_trans_time: Power_Globals.Platforms[device].silicon_rev[sili].transitionTimes.interchirpIdleTime, idle_trans_power: Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.interchirpIdleTransitionPower};
            }
            //if not enough time to enter any idle mode, then stay in active mode
            else{
                return {idle_usage:active_usage, idle_trans_time: 0, idle_trans_power:0};
            }
        }
    };
    var get_proc_usage = function(device, sili, mode, io){
        if(mode==="power"){
            return Power_Globals.Platforms[device].silicon_rev[sili].powerOptimized.processing;
        }
        else{
            return Power_Globals.Platforms[device].silicon_rev[sili].bomOptimized.processing;
        }
    };
    var get_avg_power = function(active_usage, active_time, interburst_usage, interburst_time, interburst_trans_power, interburst_trans_time, proc_usage, proc_time, interframe_usage, interframe_time, interframe_trans_power, interframe_trans_time, frametime, numbursts){
        var remainingInterburstTime = interburst_time - (interburst_trans_time*(numbursts-1)); 
        //add back whatever time was subtracted for the interburst trans time as the device has already spent that time to transition
        var remainingInterframTime  = interframe_time - (interframe_trans_time);
        return ((active_usage*active_time) + (interburst_usage*remainingInterburstTime) + (interburst_trans_power*interburst_trans_time*(numbursts-1)) + (proc_usage*(proc_time*1e-3)) + (interframe_usage*remainingInterframTime) + (interframe_trans_power*(interframe_trans_time-interburst_trans_time)))/(frametime*1e-3);
    };

    // ================================ END: GETTER FUNCTIONS  ==============================================================  

    // ================================ START: UPDATE FUNCTIONS  ============================================================ 
    var updateOutputComputation = function(powerObject){
        // Grab the self
        var Input           = powerObject.Input;
        var Intermediate    = powerObject.Intermediate;
        var Output          = powerObject.Output;
        var temp_interburst_vals = {idle_usage:0, idle_trans_time:0, idle_trans_power:0};
        var temp_interframe_vals = {idle_usage:0, idle_trans_time:0, idle_trans_power:0};
        
        Intermediate.chirps_per_burst      = get_chirps_per_burst(Input.numchirps, Input.chirps_accum);
        Intermediate.active_time_per_burst = get_active_time_per_burst(Input.rampendtime, Input.chirpidle, Intermediate.chirps_per_burst); 
        // Handle the Device and EVM Specific Parameters here
        Intermediate.active_time_seconds       = get_total_active_time(Intermediate.active_time_per_burst, Input.numbursts);
        Output.active_time                     = Intermediate.active_time_seconds*1000000;
        Intermediate.interburst_time_per_burst = get_interburst_time_per_burst(Input.burst_periodicity, Intermediate.active_time_per_burst);
        Intermediate.interburst_time_seconds   = get_interburst_time(Intermediate.interburst_time_per_burst, Input.numbursts);
        Output.interburst_time             = Intermediate.interburst_time_per_burst*1000000;
        Output.proc_time                   = Input.proc_time;
        Intermediate.interframe_time_seconds   = get_interframe_time(Input.frametime, Input.proc_time, Input.burst_periodicity, Input.numbursts, Intermediate.active_time_per_burst);
        Output.interframe_time             = Intermediate.interframe_time_seconds*1000;
        // UPDATE: Should work with different power modes and IO topologies
        Output.active_usage                = get_active_usage(Input.device, Input.num_rx, Input.num_tx, Input.sili, Input.mode, Input.io);
        temp_interburst_vals               = get_idle_usage(Input.device, Intermediate.interburst_time_per_burst, Output.active_usage, Input.sili, Input.mode, Input.io);
        Output.interburst_usage            = temp_interburst_vals.idle_usage;
        Intermediate.interburst_trans_time = temp_interburst_vals.idle_trans_time;
        Intermediate.interburst_trans_power= temp_interburst_vals.idle_trans_power;
        Output.proc_usage                  = get_proc_usage(Input.device, Input.sili, Input.mode, Input.io);
        temp_interframe_vals               = get_idle_usage(Input.device, Intermediate.interframe_time_seconds, Output.active_usage, Input.sili, Input.mode, Input.io);
        Output.interframe_usage            = temp_interframe_vals.idle_usage;
        Intermediate.interframe_trans_time = temp_interframe_vals.idle_trans_time;
        Intermediate.interframe_trans_power= temp_interframe_vals.idle_trans_power;
        Output.avg_power                   = get_avg_power(Output.active_usage, Intermediate.active_time_seconds, Output.interburst_usage, Intermediate.interburst_time_seconds, Intermediate.interburst_trans_power, Intermediate.interburst_trans_time, Output.proc_usage, Output.proc_time, Output.interframe_usage, Intermediate.interframe_time_seconds, Intermediate.interframe_trans_power, Intermediate.interframe_trans_time, Input.frametime, Input.numbursts);

    };
    var updateInput = function(changes) {
        // Grab the self:
        var power = this;

        // Log the changes that have occurred TO THE INPUT SIDE of the object:
        for (var k in changes) {
            if (changes.hasOwnProperty(k)) {
                power.Input[k] = changes[k]; 
            }
        }
        // Calculate the outputs:
        updateOutputComputation(power);
    };	
    // ================================ END: UPDATE FUNCTIONS  ============================================================== 
    // ================================ START: OBJECT EXPORTATION  ==========================================================
    mmWavePower.prototype.init                  = init;
    mmWavePower.prototype.updateInput           = updateInput;
    mmWavePower.prototype.get_allowed_evms      = get_allowed_evms;
    mmWavePower.prototype.get_allowed_silicon_rev = get_allowed_silicon_rev
    
    // export as AMD/CommonJS module or global variable
    if (typeof define === 'function' && define.amd) define('mmWavePower', function () { return mmWavePower; });
    else if (typeof module !== 'undefined') module.exports = mmWavePower;
    else if (typeof self !== 'undefined') self.mmWavePower = mmWavePower;
    else window.mmWavePower = mmWavePower;
    // ================================ END: OBJECT EXPORTATION  ============================================================
})();