/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

(function() {

    // Constructor for Util object
    function Util() {
        if (!(this instanceof Util)) 
            return new Util();
        this.init();
    };

    function init(){

        // Number and String formatting functions
        this.Format = {
            /**
             * Format a number to a certain number of digits after decimal
             * @param {*} num - some number in any format
             * @param {int} d - number of digits after decimal if num is not integer
             * @returns - num formatted according to d
             */
            sprintf: (num, d) => {
                if (Number.isInteger(num)) return num;
                else if (typeof(num) == 'number') return num.toFixed(d);
                else return num;
            },
            /**
             * Format a float to a certain number of digits after decimal
             * @param {float} x - some float in any format
             * @param {int} d - number of digits after decimal if num is not integer
             * @returns - num formatted according to d
             */
            toPrecision: (x, p) => {
                return Number(x.toFixed(p));
            },
        };
        
        // HTTP request related functions
        this.Http = {
            /**
             *  Make a request to a server and retrieve the response from the server.
             * @param {*} url 
             * @param {*} responseType 
             * @param {*} cb 
             */
            httpGet: (url, responseType, cb) => {
                var req = new XMLHttpRequest();
                req.onreadystatechange = function () {
                    if (req.readyState === 4){
                        if (req.status == 200 || (req.status === 0 && !!req.response)){
                            cb(undefined, req.response);
                        }  else {
                            cb(req.status, req);
                        }
                    }
                };
                if (responseType) {
                    req.responseType = responseType;
                }
                req.open("GET", url, true);  // async request specified by 3rd param = true
                req.send();
            },
            /**
             * Create the url needed for an http request for .json files used by this tool
             * @param {string} filename 
             * @returns 
             */
            fileToURL: (filename) => {
                var idx = location.pathname.indexOf('/index.html');
                var url = location.pathname;
                if (idx >= 0) {
                    url = location.pathname.substring(0,idx);
                }
                if (url[url.length-1] != '/') url += '/';
                url += 'app/'+filename;
                return url;
            },
        };
        
        // File I/O functions for saving and loading files into the tool
        this.FileIO = {
            /**
             * Get the unique filename for saving to system
             * @param {string} fileTypeStr 
             * @param {string} fileextension 
             * @returns 
             */
            getUniqueFileName: (fileTypeStr, fileextension) => {
                var dateAppendStr = (new Date().toISOString().replace(/[-:\.]/g, "_").replace(/[Z]/g, ""));
                return (fileTypeStr + '_' + dateAppendStr + fileextension); 
            },
        };
        
        // Cookie functions for the tool to have a system of session persistence
        this.Cookie = {
            /**
             * Create a cookie with a name and value
             * @param {string} name 
             * @param {*} value 
             */
            bakeCookie: (name, value) => {
                var cookie = [name, '=', JSON.stringify(value), '; domain=.', window.location.host.toString(), '; path=/;'].join('');
                document.cookie = cookie;
            },
            /**
             * Load a cookie with a given name
             * @param {string} name 
             * @returns 
             */
            loadCookie: (name) => {
                var result = document.cookie.match(new RegExp(name + '=([^;]+)'));
                result && (result = JSON.parse(result[1]));
                return result;
            },
            /**
             * Delete a cookie with a given name
             * @param {string} name 
             */
            deleteCookie: (name) => {
                document.cookie = [name, '=; expires=Thu, 01-Jan-1970 00:00:01 GMT; path=/; domain=.', window.location.host.toString()].join('');
            },
        };

        // HTML Element manipulation functions
        this.Html = {
            addFavicon: () => {
                var link = document.querySelector("link[rel~='icon']");
                link.type = 'image/x-icon';
                link.rel = 'shortcut icon';
                link.href = './app/favicon.ico';
                link.crossorigin = 'anonymous';
                document.getElementsByTagName('head')[0].appendChild(link);
            },
            /**
             * Change the values in a dropdown. You would typically use this conditionally based on another input changing. 
             * @param {string} widget - The dropdown whose values are being updated
             * @param {string array} values - An array of the values you want updated (the value the backend sees when a user selects something).
             * @param {string array} labels - An array of labels you want updated (what the user sees in the dropdown)
             */
            updateDroplistOptions: (widget, values, labels) => {
                widget.values = values.join('|');                               // The backend value for a selection
                widget.labels = labels ? labels.join('|') : values.join('|');   // What a user sees for a selection, if provided. 
                                                                                // Otherwise, the frontend and backend equals "values".
            },
            /**
             * Update the selected value in a droplist element.
             * @param {string} widget 
             * @param {*} newValue 
             * @returns boolean, T if value changed; F if value unchanged
             */
            updateDroplist: (widget, newValue) => {
                if (newValue) {
                    widget.selectedValue = newValue;
                    // for droplists that consist of numbers rather than strings, cast the new value to a string to find the index of it in the widget value list
                    typeof(newValue) == "string" ? widget.selectedIndex =  widget.values.split('|').indexOf(newValue) : widget.selectedIndex = widget.values.split('|').indexOf(newValue.toString());
                    return true;
                }
                return false;
            },
            /**
             * Update the value in a textbox field. If you pass prec, then will update to that level of precision
             * @param {string} widget 
             * @param {number} newValue 
             * @param {int} prec 
             */
            updateTextbox: (widget, newValue, prec) => {
                if (prec !== undefined) {
                    newValue = this.Format.sprintf(newValue, prec);
                }
                if (newValue !== undefined) {
                    widget.setText(newValue);
                }
                widget.value = newValue;
            },
            /**
             * Highlight or unhighlight an erroneous widget.
             * @param {string} widget - a gui element 
             * @param {boolean} error - true if error (apply class), false if not (remove class)
             * @param {string} errClass - the class to apply to the widget if error is true
             */
            highlightProp: (widget, error, errClass) => {
                if (!widget) return;
                if (error) widget.addClassName(errClass);
                else widget.removeClassName(errClass);
            },
            /**
             * Highlight or unhighlight an erroneous widget with a order of precedence.
             * A lower order of precedence will not overwrite a higher order of precedence.
             * 
             * Prescedence:
             * 1. error1
             * 2. warning
             * 3. conditional
             * @param {string} widget - a gui element
             * @param {boolean} error - true if error (apply class), false if not (remove class)
             * 
             */
            highlightPropWithPrecedence:(widget, error, errClass) => {
                if (!widget) return;
                if (error == 1) {
                    if (errClass == 'error1'){
                        // Clear the lower precedence highlight, if any
                        this.Html.highlightProp(widget, 0, 'warning');
                        this.Html.highlightProp(widget, 0, 'conditional');
                        // Apply this highlight
                        this.Html.highlightProp(widget, error, errClass);
                    }
                    else if (errClass == 'warning' && !widget.classList.contains('error1')){
                        // Clear the lower precedence highlight, if any
                        this.Html.highlightProp(widget, 0, 'conditional');
                        // Apply this highlight
                        this.Html.highlightProp(widget, error, errClass);
                    }
                    else if (errClass == 'conditional'&& (!widget.classList.contains('error1') && !widget.classList.contains('warning'))){
                        // Apply this highlight
                        this.Html.highlightProp(widget, error, errClass);
                    }
                }
                else if (error == 0){
                    this.Html.highlightProp(widget, error, errClass);
                }
            },
            /**
             * Select a tab in a tabContainer element.
             * @param {string} widget - a tabContainer gui element name 
             * @param {int} tabIndex - the index of the tab to select
             */
            selectTab: (widget, tabIndex) => {
                if (!widget) return;
                widget.selectedIndex = tabIndex;
            },
        };
    };
    
    // Export the Util object
    Util.prototype.init = init;

    // create a global export for this module in a variety of formats for compatibility 
    if (typeof define === 'function' && define.amd) define('Util', function () { return Util; });
    else if (typeof module !== 'undefined') module.exports = Util;
    else if (typeof self !== 'undefined') self.Util = Util;
    else window.Util = Util;
})();
